#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import click

from .error import AppError
from .lib import warn, find_base, validate_base_files, validate_base_text


def ls_dest() -> None:
    """ List destination files
    """
    try:
        _, base_txt = find_base(os.getcwd())
        base_cfg = validate_base_text(base_txt)
        click.echo("\n".join(validate_base_files(base_cfg["files"]).keys()))
    except AppError as e:
        warn(e)
        sys.exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
from pkg_resources import resource_string

import click

from .check import check
from .init import init
from .ls_dest import ls_dest
from .root_path import root_path
from .run import run

if __name__ == "__main__":
    VERSION = json.loads(
        resource_string(__name__, "../package.json"))["version"]
else:
    VERSION = json.loads(resource_string("exvar", "package.json"))["version"]


@click.group(invoke_without_command=True)
@click.pass_context
@click.option("--version", "-v", is_flag=True,
              help="Print the exvar version number and exit")
def cmd(ctx, version):
    if ctx.invoked_subcommand is None:
        if version:
            click.echo(VERSION)
        else:
            click.echo(ctx.get_help())


cmd.command()(init)
click.option(
    "--check-dest", is_flag=True, help="check the dest file")(check)
cmd.command()(check)
cmd.command()(run)
cmd.command("ls-dest")(ls_dest)
cmd.command("root-path")(root_path)


def main():
    cmd(obj={})


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

WARN_NONE_VAR = """
a var the value isn't set is found

destination file: {}
variable name: {}
""".strip()

WARN_UNUSED_VAR = """
unused variable is found

source file: {}
variable name: {}
""".strip()

WARN_DIFF_SRC_DEST = """
The content of the destination file is different from
the source text replaced variable names with variable values.
Destination files are generated automatically,
so if you add some modifications to the destination file directly,
you should add them to the source file and run `exvar run`

destination file: {}
""".strip()

WARN_UNKNOWN_VAR_EXIST = """
the variable "{}" exists in the .exvar.yml
but doesn't exist in the .exvar.base.yml

destination file: {}
variable name: {}
""".strip()

WARN_UNKNOWN_FILE_EXIST = """
the destination file "{}" exists in the .exvar.yml
but doesn't exist in the .exvar.base.yml
""".strip()

BASE_FILE_NOT_FOUND = """
.exvar.base.yml is not found
""".strip()

INVALID_BASE_CFG = """
the format of .exvar.base.yml is invalid
""".strip()

INVALID_USER_CFG = """
the format of .exvar.yml is invalid
""".strip()

INVALID_VAR = """
the variable value must be string or int or null.
""".strip()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .message import (BASE_FILE_NOT_FOUND, INVALID_BASE_CFG, INVALID_USER_CFG,
                      INVALID_VAR)


class AppError(Exception):
    def __init__(self, msg=None):
        if msg is not None:
            self.message = msg

    def __str__(self):
        return self.message


class BaseFileNotFoundError(AppError):
    message = BASE_FILE_NOT_FOUND


class InvalidBaseCfgError(AppError):
    message = INVALID_BASE_CFG


class InvalidUserCfgError(AppError):
    message = INVALID_USER_CFG


class InvalidVarError(AppError):
    message = INVALID_VAR



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import shutil

from .cnst import BASE_FILE, USER_FILE, BASE_TMPL, USER_TMPL


def _init(dirname: str, src: str, dest: str) -> None:
    if not os.path.lexists(dest):
        shutil.copyfile(os.path.normpath(os.path.join(dirname, src)), dest)


def init() -> None:
    """ Create .exvar.base.yml and .exvar.yml
    """
    dirname = os.path.dirname(__file__)
    _init(dirname, BASE_TMPL, BASE_FILE)
    _init(dirname, USER_TMPL, USER_FILE)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import click

from .error import AppError
from .lib import warn, find_base


def root_path() -> None:
    """ Print the absolute path of the parent directory of the base file.
    """
    try:
        click.echo(find_base(os.getcwd())[0])
    except AppError as e:
        warn(e)
        sys.exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

from .error import AppError
from .lib import warn, execute


def check(check_dest) -> None:
    """ Validate the base file and user file and
    source files and destination files.
    """
    try:
        if not execute(is_check=True, is_check_dest=check_dest):
            sys.exit(1)
    except AppError as e:
        warn(e)
        sys.exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import os
import sys

import yaml

from .cnst import BASE_FILE, USER_FILE
from .error import (
    BaseFileNotFoundError, InvalidBaseCfgError,
    InvalidUserCfgError, InvalidVarError)
from .message import (
    WARN_NONE_VAR, WARN_UNUSED_VAR, WARN_DIFF_SRC_DEST,
    WARN_UNKNOWN_FILE_EXIST, WARN_UNKNOWN_VAR_EXIST, INVALID_BASE_CFG,
    INVALID_USER_CFG)


def warn(msg) -> None:
    sys.stderr.write("[exvar][Warning] {}\n".format(msg))


def find_base(cwd: str):
    while True:
        try:
            base_cfg_path = os.path.join(cwd, BASE_FILE)
            with open(base_cfg_path) as r:
                return (cwd, r.read())
        except FileNotFoundError:
            if cwd == "/":
                raise BaseFileNotFoundError()
            cwd = os.path.dirname(cwd)


def get_str(value) -> str:
    if value is None:
        return ""
    elif isinstance(value, (str, int)):
        return str(value)
    else:
        raise InvalidVarError()


def execute(is_run=False, is_check=False, is_check_dest=False) -> bool:
    """
    """
    result = True
    pardir, base_txt = find_base(os.getcwd())
    base_cfg = validate_base_text(base_txt)
    if "config" not in base_cfg:
        raise InvalidBaseCfgError()
    if "default_prefix" not in base_cfg["config"]:
        raise InvalidBaseCfgError()
    if "default_suffix" not in base_cfg["config"]:
        raise InvalidBaseCfgError()
    default_prefix = base_cfg["config"].get("default_prefix")
    default_suffix = base_cfg["config"].get("default_suffix")
    if default_prefix is None:
        default_prefix = ""
    if default_suffix is None:
        default_suffix = ""
    try:
        with open(os.path.join(pardir, USER_FILE)) as r:
            user_cfg = yaml.load(r.read())
    except FileNotFoundError:
        # user config file is not found
        user_cfg = {"files": {}}
    except yaml.parser.ParserError as e:
        raise InvalidUserCfgError("{}\n{}".format(INVALID_USER_CFG, e))
    if not isinstance(user_cfg, dict):
        raise InvalidUserCfgError()
    if "files" not in user_cfg:
        raise InvalidUserCfgError()
    user_files = user_cfg["files"]
    if user_files is None:
        user_files = {}
    elif not isinstance(user_files, dict):
        raise InvalidUserCfgError()
    base_files = validate_base_files(base_cfg["files"])
    for dest, base_file in base_files.items():
        user_file = user_files.get(dest, {})
        if not isinstance(user_file, dict):
            raise InvalidUserCfgError()
        src = base_file.get("src", "{}{}{}".format(
            default_prefix, dest, default_suffix))
        src_path = os.path.join(pardir, src)
        try:
            with open(src_path) as r:
                src_text = r.read()
        except FileNotFoundError:
            # source file is not found
            warn("can't read {}".format(src_path))
            result = False
            continue
        for var_name, base_var in base_file.get("vars", {}).items():
            if var_name in user_file:
                user_var = user_file[var_name]
                if "value" in user_var:
                    value = get_str(user_var["value"])
                else:
                    if "value" in base_var:
                        value = get_str(base_var["value"])
                    else:
                        warn(WARN_NONE_VAR.format(dest, var_name))
                        result = False
                        continue
            else:
                if "value" in base_var:
                    value = get_str(base_var["value"])
                else:
                    warn(WARN_NONE_VAR.format(dest, var_name))
                    result = False
                    continue
            new_text = src_text.replace(var_name, value)
            if is_check and src_text == new_text:
                # the var is not used
                warn(WARN_UNUSED_VAR.format(src_path, var_name))
                result = False
                continue
            src_text = new_text
        if is_check_dest:
            try:
                with open(os.path.join(pardir, dest)) as r:
                    dest_text = r.read()
                if src_text != dest_text:
                    # warning when check command
                    warn(WARN_DIFF_SRC_DEST.format(dest))
                    result = False
                    if is_run:
                        with open(os.path.join(pardir, dest), "w") as w:
                            w.write(src_text)
            except FileNotFoundError:
                # dest file is not found
                if is_run:
                    with open(os.path.join(pardir, dest), "w") as w:
                        w.write(src_text)
        else:
            if is_run:
                with open(os.path.join(pardir, dest), "w") as w:
                    w.write(src_text)
        if is_check:
            for var_name, user_var in user_file.items():
                if var_name not in base_files:
                    warn(WARN_UNKNOWN_VAR_EXIST.format(
                        var_name, dest, var_name))
                    result = False
    if is_check:
        for dest, user_file in user_files.items():
            if dest not in base_files:
                # unknown src exists in user config
                warn(WARN_UNKNOWN_FILE_EXIST.format(dest))
                result = False
    return result


def validate_base_text(text):
    try:
        base_cfg = yaml.load(text)
    except yaml.parser.ParserError as e:
        raise InvalidBaseCfgError("{}\n{}".format(INVALID_BASE_CFG, e))
    if not isinstance(base_cfg, dict):
        raise InvalidBaseCfgError()
    if "files" not in base_cfg:
        raise InvalidBaseCfgError()
    return base_cfg


def validate_base_files(base_files):
    if base_files is None:
        return {}
    elif not isinstance(base_files, dict):
        raise InvalidBaseCfgError()
    return base_files



import json
import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), "package.json")) as r:
    data = json.load(r)

with open(os.path.join(os.path.dirname(__file__), "README.rst")) as r:
    LONG_DESCRIPTION = r.read()

setup(
    name="exvar",
    version=data["version"],

    packages=["exvar", "exvar.src"],
    package_dir={
        "exvar": ".",
        "exvar.src": "src",
    },
    package_data={
        "exvar": ["package.json"],
        "exvar.src": ["exvar.base.yml", "exvar.yml"],
    },
    install_requires=["click", "pyyaml"],
    zip_safe=True,
    entry_points={
        "console_scripts": ["exvar=exvar.src.main:main"],
    },

    description=data["description"],
    long_description=LONG_DESCRIPTION,
    url=data["repository"],
    license=data["license"],
    author="Suzuki Shunsuke",
    author_email="suzuki.shunsuke.1989@gmail.com"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

from .error import AppError
from .lib import warn, execute


def run() -> None:
    """ Create or update dest files
    """
    try:
        execute(is_run=True)
    except AppError as e:
        warn(e)
        sys.exit(1)
